@@ -1,12 +1,41 @@
-# -*- coding: utf-8 -*-
+# Copyright (c) 2013, Mahmoud Hashemi
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
+#
+#    * Redistributions of source code must retain the above copyright
+#      notice, this list of conditions and the following disclaimer.
+#
+#    * Redistributions in binary form must reproduce the above
+#      copyright notice, this list of conditions and the following
+#      disclaimer in the documentation and/or other materials provided
+#      with the distribution.
+#
+#    * The names of the contributors may not be used to endorse or
+#      promote products derived from this software without specific
+#      prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 """So much practical programming involves string manipulation, which
 Python readily accommodates. Still, there are dozens of basic and
 common capabilities missing from the standard library, several of them
 provided by ``strutils``.
 """
 
-from __future__ import print_function
 
+import builtins
 import re
 import sys
 import uuid
@@ -14,27 +43,11 @@ import zlib
 import string
 import unicodedata
 import collections
+from collections.abc import Mapping
 from gzip import GzipFile
-
-try:
-    from cStringIO import cStringIO as StringIO
-except ImportError:
-    from io import BytesIO as StringIO
-
-try:
-    from collections.abc import Mapping
-except ImportError:
-    from collections import Mapping
-
-try:
-    unicode, str, bytes, basestring = unicode, str, str, basestring
-    from HTMLParser import HTMLParser
-    import htmlentitydefs
-except NameError:  # basestring not defined in Python 3
-    unicode, str, bytes, basestring = str, bytes, bytes, (str, bytes)
-    unichr = chr
-    from html.parser import HTMLParser
-    from html import entities as htmlentitydefs
+from html.parser import HTMLParser
+from html import entities as htmlentitydefs
+from io import BytesIO as StringIO
 
 
 __all__ = ['camel2under', 'under2camel', 'slugify', 'split_punct_ws',
@@ -42,7 +55,9 @@ __all__ = ['camel2under', 'under2camel', 'slugify', 'split_punct_ws',
            'asciify', 'is_ascii', 'is_uuid', 'html2text', 'strip_ansi',
            'bytes2human', 'find_hashtags', 'a10n', 'gzip_bytes', 'gunzip_bytes',
            'iter_splitlines', 'indent', 'escape_shell_args',
-           'args2cmd', 'args2sh', 'parse_int_list', 'format_int_list']
+           'args2cmd', 'args2sh', 'parse_int_list', 'format_int_list',
+           'complement_int_list', 'int_ranges_from_int_list', 'MultiReplace',
+           'multi_replace', 'unwrap_text', 'removeprefix']
 
 
 _punct_ws_str = string.punctuation + string.whitespace
@@ -122,8 +137,8 @@ def unit_len(sized_iterable, unit_noun='item'):  # TODO: len_units()/unitize()?
     count = len(sized_iterable)
     units = cardinalize(unit_noun, count)
     if count:
-        return u'%s %s' % (count, units)
-    return u'No %s' % (units,)
+        return f'{count} {units}'
+    return f'No {units}'
 
 
 _ORDINAL_MAP = {'1': 'st',
@@ -149,7 +164,7 @@ def ordinalize(number, ext_only=False):
     >>> print(ordinalize(1515))
     1515th
     """
-    numstr, ext = unicode(number), ''
+    numstr, ext = str(number), ''
     if numstr and numstr[-1] in string.digits:
         try:
             # first check for teens
@@ -281,19 +296,19 @@ _IRR_S2P = {'addendum': 'addenda', 'alga': 'algae', 'alumna': 'alumnae',
             'ox': 'oxen', 'paralysis': 'paralyses', 'parenthesis': 'parentheses',
             'person': 'people', 'phenomenon': 'phenomena', 'potato': 'potatoes',
             'radius': 'radii', 'scarf': 'scarves', 'scissors': 'scissors',
-            'self': 'selves', 'series': 'series', 'sheep': 'sheep',
-            'shelf': 'shelves', 'species': 'species', 'stimulus': 'stimuli',
-            'stratum': 'strata', 'syllabus': 'syllabi', 'symposium': 'symposia',
-            'synopsis': 'synopses', 'synthesis': 'syntheses', 'tableau': 'tableaux',
-            'that': 'those', 'thesis': 'theses', 'thief': 'thieves',
-            'this': 'these', 'tomato': 'tomatoes', 'tooth': 'teeth',
-            'torpedo': 'torpedoes', 'vertebra': 'vertebrae', 'veto': 'vetoes',
-            'vita': 'vitae', 'watch': 'watches', 'wife': 'wives',
+            'self': 'selves', 'sense': 'senses', 'series': 'series', 'sheep':
+            'sheep', 'shelf': 'shelves', 'species': 'species', 'stimulus':
+            'stimuli', 'stratum': 'strata', 'syllabus': 'syllabi', 'symposium':
+            'symposia', 'synopsis': 'synopses', 'synthesis': 'syntheses',
+            'tableau': 'tableaux', 'that': 'those', 'thesis': 'theses',
+            'thief': 'thieves', 'this': 'these', 'tomato': 'tomatoes', 'tooth':
+            'teeth', 'torpedo': 'torpedoes', 'vertebra': 'vertebrae', 'veto':
+            'vetoes', 'vita': 'vitae', 'watch': 'watches', 'wife': 'wives',
             'wolf': 'wolves', 'woman': 'women'}
 
 
 # Reverse index of the above
-_IRR_P2S = dict([(v, k) for k, v in _IRR_S2P.items()])
+_IRR_P2S = {v: k for k, v in _IRR_S2P.items()}
 
 HASHTAG_RE = re.compile(r"(?:^|\s)[＃#]{1}(\w+)", re.UNICODE)
 
@@ -331,12 +346,25 @@ def a10n(string):
     """
     if len(string) < 3:
         return string
-    return '%s%s%s' % (string[0], len(string[1:-1]), string[-1])
+    return f'{string[0]}{len(string[1:-1])}{string[-1]}'
 
 
-ANSI_ESCAPE_BEGIN = '\x1b['
-ANSI_TERMINATORS = ('H', 'f', 'A', 'B', 'C', 'D', 'R', 's', 'u', 'J',
-                    'K', 'h', 'l', 'p', 'm')
+# Based on https://en.wikipedia.org/wiki/ANSI_escape_code#Escape_sequences
+ANSI_SEQUENCES = re.compile(r'''
+    \x1B            # Sequence starts with ESC, i.e. hex 0x1B
+    (?:
+        [@-Z\\-_]   # Second byte:
+                    #   all 0x40–0x5F range but CSI char, i.e ASCII @A–Z\]^_
+    |               # Or
+        \[          # CSI sequences, starting with [
+        [0-?]*      # Parameter bytes:
+                    #   range 0x30–0x3F, ASCII 0–9:;<=>?
+        [ -/]*      # Intermediate bytes:
+                    #   range 0x20–0x2F, ASCII space and !"#$%&'()*+,-./
+        [@-~]       # Final byte
+                    #   range 0x40–0x7E, ASCII @A–Z[\]^_`a–z{|}~
+    )
+''', re.VERBOSE)
 
 
 def strip_ansi(text):
@@ -344,33 +372,37 @@ def strip_ansi(text):
     time when a log or redirected output accidentally captures console
     color codes and the like.
 
-    >>> strip_ansi('\x1b[0m\x1b[1;36mart\x1b[46;34m\xdc')
+    >>> strip_ansi('\x1b[0m\x1b[1;36mart\x1b[46;34m')
     'art'
 
-    The test above is an excerpt from ANSI art on
-    `sixteencolors.net`_. This function does not interpret or render
-    ANSI art, but you can do so with `ansi2img`_ or `escapes.js`_.
+    Supports str, bytes and bytearray content as input. Returns the
+    same type as the input.
+
+    There's a lot of ANSI art available for testing on `sixteencolors.net`_.
+    This function does not interpret or render ANSI art, but you can do so with
+    `ansi2img`_ or `escapes.js`_.
 
     .. _sixteencolors.net: http://sixteencolors.net
     .. _ansi2img: http://www.bedroomlan.org/projects/ansi2img
     .. _escapes.js: https://github.com/atdt/escapes.js
     """
     # TODO: move to cliutils.py
-    nansi, keep, i, text_len = [], True, 0, len(text)
-    while i < text_len:
-        if not keep and text[i] in ANSI_TERMINATORS:
-            keep = True
-        elif keep:
-            keep_end_i = text.find(ANSI_ESCAPE_BEGIN, i)
-            if keep_end_i < 0:
-                break
-            else:
-                nansi.append(text[i:keep_end_i])
-                i, keep = keep_end_i, False
-        i += 1
-    if not nansi:
-        return text
-    return type(text)().join(nansi)  # attempted unicode + str support
+
+    # Transform any ASCII-like content to unicode to allow regex to match, and
+    # save input type for later.
+    target_type = None
+    # Unicode type aliased to str is code-smell for Boltons in Python 3 env.
+    if isinstance(text, (bytes, bytearray)):
+        target_type = type(text)
+        text = text.decode('utf-8')
+
+    cleaned = ANSI_SEQUENCES.sub('', text)
+
+    # Transform back the result to the same bytearray type provided by the user.
+    if target_type and target_type != type(cleaned):
+        cleaned = target_type(cleaned, 'utf-8')
+
+    return cleaned
 
 
 def asciify(text, ignore=False):
@@ -383,9 +415,9 @@ def asciify(text, ignore=False):
     **with** unicode, not against it.
 
     Args:
-        text (str or unicode): The string to be asciified.
+        text (str): The string to be asciified.
         ignore (bool): Configures final encoding to ignore remaining
-            unasciified unicode instead of replacing it.
+            unasciified string instead of replacing it.
 
     >>> asciify('Beyoncé') == b'Beyonce'
     True
@@ -407,18 +439,18 @@ def asciify(text, ignore=False):
 
 
 def is_ascii(text):
-    """Check if a unicode or bytestring, *text*, is composed of ascii
+    """Check if a string or bytestring, *text*, is composed of ascii
     characters only. Raises :exc:`ValueError` if argument is not text.
 
     Args:
-        text (str or unicode): The string to be checked.
+        text (str): The string to be checked.
 
     >>> is_ascii('Beyoncé')
     False
     >>> is_ascii('Beyonce')
     True
     """
-    if isinstance(text, unicode):
+    if isinstance(text, str):
         try:
             text.encode('ascii')
         except UnicodeEncodeError:
@@ -440,7 +472,7 @@ class DeaccenterDict(dict):
         if ch is not None:
             return ch
         try:
-            de = unicodedata.decomposition(unichr(key))
+            de = unicodedata.decomposition(chr(key))
             p1, _, p2 = de.rpartition(' ')
             if int(p2, 16) == 0x308:
                 ch = self.get(key)
@@ -451,72 +483,59 @@ class DeaccenterDict(dict):
         self[key] = ch
         return ch
 
-    try:
-        from collections import defaultdict
-    except ImportError:
-        # no defaultdict means that __missing__ isn't supported in
-        # this version of python, so we define __getitem__
-        def __getitem__(self, key):
-            try:
-                return super(DeaccenterDict, self).__getitem__(key)
-            except KeyError:
-                return self.__missing__(key)
-    else:
-        del defaultdict
-
 
 # http://chmullig.com/2009/12/python-unicode-ascii-ifier/
 # For something more complete, investigate the unidecode
 # or isounidecode packages, which are capable of performing
 # crude transliteration.
 _BASE_DEACCENT_MAP = {
-    0xc6: u"AE", # Æ LATIN CAPITAL LETTER AE
-    0xd0: u"D",  # Ð LATIN CAPITAL LETTER ETH
-    0xd8: u"OE", # Ø LATIN CAPITAL LETTER O WITH STROKE
-    0xde: u"Th", # Þ LATIN CAPITAL LETTER THORN
-    0xc4: u'Ae', # Ä LATIN CAPITAL LETTER A WITH DIAERESIS
-    0xd6: u'Oe', # Ö LATIN CAPITAL LETTER O WITH DIAERESIS
-    0xdc: u'Ue', # Ü LATIN CAPITAL LETTER U WITH DIAERESIS
-    0xc0: u"A",  # À LATIN CAPITAL LETTER A WITH GRAVE
-    0xc1: u"A",  # Á LATIN CAPITAL LETTER A WITH ACUTE
-    0xc3: u"A",  # Ã LATIN CAPITAL LETTER A WITH TILDE
-    0xc7: u"C",  # Ç LATIN CAPITAL LETTER C WITH CEDILLA
-    0xc8: u"E",  # È LATIN CAPITAL LETTER E WITH GRAVE
-    0xc9: u"E",  # É LATIN CAPITAL LETTER E WITH ACUTE
-    0xca: u"E",  # Ê LATIN CAPITAL LETTER E WITH CIRCUMFLEX
-    0xcc: u"I",  # Ì LATIN CAPITAL LETTER I WITH GRAVE
-    0xcd: u"I",  # Í LATIN CAPITAL LETTER I WITH ACUTE
-    0xd2: u"O",  # Ò LATIN CAPITAL LETTER O WITH GRAVE
-    0xd3: u"O",  # Ó LATIN CAPITAL LETTER O WITH ACUTE
-    0xd5: u"O",  # Õ LATIN CAPITAL LETTER O WITH TILDE
-    0xd9: u"U",  # Ù LATIN CAPITAL LETTER U WITH GRAVE
-    0xda: u"U",  # Ú LATIN CAPITAL LETTER U WITH ACUTE
-    0xdf: u"ss", # ß LATIN SMALL LETTER SHARP S
-    0xe6: u"ae", # æ LATIN SMALL LETTER AE
-    0xf0: u"d",  # ð LATIN SMALL LETTER ETH
-    0xf8: u"oe", # ø LATIN SMALL LETTER O WITH STROKE
-    0xfe: u"th", # þ LATIN SMALL LETTER THORN,
-    0xe4: u'ae', # ä LATIN SMALL LETTER A WITH DIAERESIS
-    0xf6: u'oe', # ö LATIN SMALL LETTER O WITH DIAERESIS
-    0xfc: u'ue', # ü LATIN SMALL LETTER U WITH DIAERESIS
-    0xe0: u"a",  # à LATIN SMALL LETTER A WITH GRAVE
-    0xe1: u"a",  # á LATIN SMALL LETTER A WITH ACUTE
-    0xe3: u"a",  # ã LATIN SMALL LETTER A WITH TILDE
-    0xe7: u"c",  # ç LATIN SMALL LETTER C WITH CEDILLA
-    0xe8: u"e",  # è LATIN SMALL LETTER E WITH GRAVE
-    0xe9: u"e",  # é LATIN SMALL LETTER E WITH ACUTE
-    0xea: u"e",  # ê LATIN SMALL LETTER E WITH CIRCUMFLEX
-    0xec: u"i",  # ì LATIN SMALL LETTER I WITH GRAVE
-    0xed: u"i",  # í LATIN SMALL LETTER I WITH ACUTE
-    0xf2: u"o",  # ò LATIN SMALL LETTER O WITH GRAVE
-    0xf3: u"o",  # ó LATIN SMALL LETTER O WITH ACUTE
-    0xf5: u"o",  # õ LATIN SMALL LETTER O WITH TILDE
-    0xf9: u"u",  # ù LATIN SMALL LETTER U WITH GRAVE
-    0xfa: u"u",  # ú LATIN SMALL LETTER U WITH ACUTE
-    0x2018: u"'",  # ‘ LEFT SINGLE QUOTATION MARK
-    0x2019: u"'",  # ’ RIGHT SINGLE QUOTATION MARK
-    0x201c: u'"',  # “ LEFT DOUBLE QUOTATION MARK
-    0x201d: u'"',  # ” RIGHT DOUBLE QUOTATION MARK
+    0xc6: "AE", # Æ LATIN CAPITAL LETTER AE
+    0xd0: "D",  # Ð LATIN CAPITAL LETTER ETH
+    0xd8: "OE", # Ø LATIN CAPITAL LETTER O WITH STROKE
+    0xde: "Th", # Þ LATIN CAPITAL LETTER THORN
+    0xc4: 'Ae', # Ä LATIN CAPITAL LETTER A WITH DIAERESIS
+    0xd6: 'Oe', # Ö LATIN CAPITAL LETTER O WITH DIAERESIS
+    0xdc: 'Ue', # Ü LATIN CAPITAL LETTER U WITH DIAERESIS
+    0xc0: "A",  # À LATIN CAPITAL LETTER A WITH GRAVE
+    0xc1: "A",  # Á LATIN CAPITAL LETTER A WITH ACUTE
+    0xc3: "A",  # Ã LATIN CAPITAL LETTER A WITH TILDE
+    0xc7: "C",  # Ç LATIN CAPITAL LETTER C WITH CEDILLA
+    0xc8: "E",  # È LATIN CAPITAL LETTER E WITH GRAVE
+    0xc9: "E",  # É LATIN CAPITAL LETTER E WITH ACUTE
+    0xca: "E",  # Ê LATIN CAPITAL LETTER E WITH CIRCUMFLEX
+    0xcc: "I",  # Ì LATIN CAPITAL LETTER I WITH GRAVE
+    0xcd: "I",  # Í LATIN CAPITAL LETTER I WITH ACUTE
+    0xd2: "O",  # Ò LATIN CAPITAL LETTER O WITH GRAVE
+    0xd3: "O",  # Ó LATIN CAPITAL LETTER O WITH ACUTE
+    0xd5: "O",  # Õ LATIN CAPITAL LETTER O WITH TILDE
+    0xd9: "U",  # Ù LATIN CAPITAL LETTER U WITH GRAVE
+    0xda: "U",  # Ú LATIN CAPITAL LETTER U WITH ACUTE
+    0xdf: "ss", # ß LATIN SMALL LETTER SHARP S
+    0xe6: "ae", # æ LATIN SMALL LETTER AE
+    0xf0: "d",  # ð LATIN SMALL LETTER ETH
+    0xf8: "oe", # ø LATIN SMALL LETTER O WITH STROKE
+    0xfe: "th", # þ LATIN SMALL LETTER THORN,
+    0xe4: 'ae', # ä LATIN SMALL LETTER A WITH DIAERESIS
+    0xf6: 'oe', # ö LATIN SMALL LETTER O WITH DIAERESIS
+    0xfc: 'ue', # ü LATIN SMALL LETTER U WITH DIAERESIS
+    0xe0: "a",  # à LATIN SMALL LETTER A WITH GRAVE
+    0xe1: "a",  # á LATIN SMALL LETTER A WITH ACUTE
+    0xe3: "a",  # ã LATIN SMALL LETTER A WITH TILDE
+    0xe7: "c",  # ç LATIN SMALL LETTER C WITH CEDILLA
+    0xe8: "e",  # è LATIN SMALL LETTER E WITH GRAVE
+    0xe9: "e",  # é LATIN SMALL LETTER E WITH ACUTE
+    0xea: "e",  # ê LATIN SMALL LETTER E WITH CIRCUMFLEX
+    0xec: "i",  # ì LATIN SMALL LETTER I WITH GRAVE
+    0xed: "i",  # í LATIN SMALL LETTER I WITH ACUTE
+    0xf2: "o",  # ò LATIN SMALL LETTER O WITH GRAVE
+    0xf3: "o",  # ó LATIN SMALL LETTER O WITH ACUTE
+    0xf5: "o",  # õ LATIN SMALL LETTER O WITH TILDE
+    0xf9: "u",  # ù LATIN SMALL LETTER U WITH GRAVE
+    0xfa: "u",  # ú LATIN SMALL LETTER U WITH ACUTE
+    0x2018: "'",  # ‘ LEFT SINGLE QUOTATION MARK
+    0x2019: "'",  # ’ RIGHT SINGLE QUOTATION MARK
+    0x201c: '"',  # “ LEFT DOUBLE QUOTATION MARK
+    0x201d: '"',  # ” RIGHT DOUBLE QUOTATION MARK
     }
 
 
@@ -561,22 +580,22 @@ class HTMLTextExtractor(HTMLParser):
         self.result.append(d)
 
     def handle_charref(self, number):
-        if number[0] == u'x' or number[0] == u'X':
+        if number[0] == 'x' or number[0] == 'X':
             codepoint = int(number[1:], 16)
         else:
             codepoint = int(number)
-        self.result.append(unichr(codepoint))
+        self.result.append(chr(codepoint))
 
     def handle_entityref(self, name):
         try:
             codepoint = htmlentitydefs.name2codepoint[name]
         except KeyError:
-            self.result.append(u'&' + name + u';')
+            self.result.append('&' + name + ';')
         else:
-            self.result.append(unichr(codepoint))
+            self.result.append(chr(codepoint))
 
     def get_text(self):
-        return u''.join(self.result)
+        return ''.join(self.result)
 
 
 def html2text(html):
@@ -928,7 +947,7 @@ def format_int_list(int_list, delim=',', range_delim='-', delim_space=False):
 
             # Current value is non-contiguous.
             elif delta > 1:
-                range_substr = '{0:d}{1}{2:d}'.format(min(contig_range),
+                range_substr = '{:d}{}{:d}'.format(min(contig_range),
                                                       range_delim,
                                                       max(contig_range))
                 output.append(range_substr)
@@ -949,7 +968,7 @@ def format_int_list(int_list, delim=',', range_delim='-', delim_space=False):
 
             # Current value is non-contiguous.
             elif delta > 1:
-                output.append('{0:d}'.format(contig_range.popleft()))
+                output.append(f'{contig_range.popleft():d}')
                 contig_range.append(x)
 
             # Current value repeated.
@@ -961,12 +980,12 @@ def format_int_list(int_list, delim=',', range_delim='-', delim_space=False):
 
         # Last value is non-contiguous.
         if len(contig_range) == 1:
-            output.append('{0:d}'.format(contig_range.popleft()))
+            output.append(f'{contig_range.popleft():d}')
             contig_range.clear()
 
         # Last value is part of contiguous range.
         elif len(contig_range) > 1:
-            range_substr = '{0:d}{1}{2:d}'.format(min(contig_range),
+            range_substr = '{:d}{}{:d}'.format(min(contig_range),
                                                   range_delim,
                                                   max(contig_range))
             output.append(range_substr)
@@ -980,7 +999,130 @@ def format_int_list(int_list, delim=',', range_delim='-', delim_space=False):
     return output_str
 
 
-class MultiReplace(object):
+def complement_int_list(
+        range_string, range_start=0, range_end=None,
+        delim=',', range_delim='-'):
+    """ Returns range string that is the complement of the one provided as
+    *range_string* parameter.
+
+    These range strings are of the kind produce by :func:`format_int_list`, and
+    parseable by :func:`parse_int_list`.
+
+    Args:
+        range_string (str): String of comma separated positive integers or
+           ranges (e.g. '1,2,4-6,8'). Typical of a custom page range string
+           used in printer dialogs.
+        range_start (int): A positive integer from which to start the resulting
+           range. Value is inclusive. Defaults to ``0``.
+        range_end (int): A positive integer from which the produced range is
+           stopped. Value is exclusive. Defaults to the maximum value found in
+           the provided ``range_string``.
+        delim (char): Defaults to ','. Separates integers and contiguous ranges
+           of integers.
+        range_delim (char): Defaults to '-'. Indicates a contiguous range of
+           integers.
+
+    >>> complement_int_list('1,3,5-8,10-11,15')
+    '0,2,4,9,12-14'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_start=0)
+    '0,2,4,9,12-14'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_start=1)
+    '2,4,9,12-14'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_start=2)
+    '2,4,9,12-14'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_start=3)
+    '4,9,12-14'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_end=15)
+    '0,2,4,9,12-14'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_end=14)
+    '0,2,4,9,12-13'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_end=13)
+    '0,2,4,9,12'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_end=20)
+    '0,2,4,9,12-14,16-19'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_end=0)
+    ''
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_start=-1)
+    '0,2,4,9,12-14'
+
+    >>> complement_int_list('1,3,5-8,10-11,15', range_end=-1)
+    ''
+
+    >>> complement_int_list('1,3,5-8', range_start=1, range_end=1)
+    ''
+
+    >>> complement_int_list('1,3,5-8', range_start=2, range_end=2)
+    ''
+
+    >>> complement_int_list('1,3,5-8', range_start=2, range_end=3)
+    '2'
+
+    >>> complement_int_list('1,3,5-8', range_start=-10, range_end=-5)
+    ''
+
+    >>> complement_int_list('1,3,5-8', range_start=20, range_end=10)
+    ''
+
+    >>> complement_int_list('')
+    ''
+    """
+    int_list = set(parse_int_list(range_string, delim, range_delim))
+    if range_end is None:
+        if int_list:
+            range_end = max(int_list) + 1
+        else:
+            range_end = range_start
+    complement_values = set(
+        range(range_end)) - int_list - set(range(range_start))
+    return format_int_list(complement_values, delim, range_delim)
+
+
+def int_ranges_from_int_list(range_string, delim=',', range_delim='-'):
+    """ Transform a string of ranges (*range_string*) into a tuple of tuples.
+
+    Args:
+        range_string (str): String of comma separated positive integers or
+           ranges (e.g. '1,2,4-6,8'). Typical of a custom page range string
+           used in printer dialogs.
+        delim (char): Defaults to ','. Separates integers and contiguous ranges
+           of integers.
+        range_delim (char): Defaults to '-'. Indicates a contiguous range of
+           integers.
+
+    >>> int_ranges_from_int_list('1,3,5-8,10-11,15')
+    ((1, 1), (3, 3), (5, 8), (10, 11), (15, 15))
+
+    >>> int_ranges_from_int_list('1')
+    ((1, 1),)
+
+    >>> int_ranges_from_int_list('')
+    ()
+    """
+    int_tuples = []
+    # Normalize the range string to our internal format for processing.
+    range_string = format_int_list(
+        parse_int_list(range_string, delim, range_delim))
+    if range_string:
+        for bounds in range_string.split(','):
+            if '-' in bounds:
+                start, end = bounds.split('-')
+            else:
+                start, end = bounds, bounds
+            int_tuples.append((int(start), int(end)))
+    return tuple(int_tuples)
+
+
+class MultiReplace:
     """
     MultiReplace is a tool for doing multiple find/replace actions in one pass.
 
@@ -998,8 +1140,8 @@ class MultiReplace(object):
 
     Dictionary Usage::
 
-        from lrmslib import stringutils
-        s = stringutils.MultiReplace({
+        from boltons import strutils
+        s = strutils.MultiReplace({
             'foo': 'zoo',
             'cat': 'hat',
             'bat': 'kraken'
@@ -1009,8 +1151,8 @@ class MultiReplace(object):
 
     Iterable Usage::
 
-        from lrmslib import stringutils
-        s = stringutils.MultiReplace([
+        from boltons import strutils
+        s = strutils.MultiReplace([
             ('foo', 'zoo'),
             ('cat', 'hat'),
             ('bat', 'kraken)'
@@ -1051,8 +1193,8 @@ class MultiReplace(object):
             sub_map = sub_map.items()
 
         for idx, vals in enumerate(sub_map):
-            group_name = 'group{0}'.format(idx)
-            if isinstance(vals[0], basestring):
+            group_name = f'group{idx}'
+            if isinstance(vals[0], str):
                 # If we're not treating input strings like a regex, escape it
                 if not options['regex']:
                     exp = re.escape(vals[0])
@@ -1061,10 +1203,7 @@ class MultiReplace(object):
             else:
                 exp = vals[0].pattern
 
-            regex_values.append('(?P<{0}>{1})'.format(
-                group_name,
-                exp
-            ))
+            regex_values.append(f'(?P<{group_name}>{exp})')
             self.group_map[group_name] = vals[1]
 
         self.combined_pattern = re.compile(
@@ -1089,6 +1228,62 @@ class MultiReplace(object):
 
 
 def multi_replace(text, sub_map, **kwargs):
-    """Shortcut function to invoke multi-replace in a single command."""
+    """
+    Shortcut function to invoke MultiReplace in a single call.
+
+    Example Usage::
+
+        from boltons.strutils import multi_replace
+        new = multi_replace(
+            'The foo bar cat ate a bat',
+            {'foo': 'zoo', 'cat': 'hat', 'bat': 'kraken'}
+        )
+        new == 'The zoo bar hat ate a kraken'
+    """
     m = MultiReplace(sub_map, **kwargs)
     return m.sub(text)
+
+
+def unwrap_text(text, ending='\n\n'):
+    r"""
+    Unwrap text, the natural complement to :func:`textwrap.wrap`.
+
+    >>> text = "Short \n lines  \nwrapped\nsmall.\n\nAnother\nparagraph."
+    >>> unwrap_text(text)
+    'Short lines wrapped small.\n\nAnother paragraph.'
+
+    Args:
+       text: A string to unwrap.
+       ending (str): The string to join all unwrapped paragraphs
+          by. Pass ``None`` to get the list. Defaults to '\n\n' for
+          compatibility with Markdown and RST.
+
+    """
+    all_grafs = []
+    cur_graf = []
+    for line in text.splitlines():
+        line = line.strip()
+        if line:
+            cur_graf.append(line)
+        else:
+            all_grafs.append(' '.join(cur_graf))
+            cur_graf = []
+    if cur_graf:
+        all_grafs.append(' '.join(cur_graf))
+    if ending is None:
+        return all_grafs
+    return ending.join(all_grafs)
+
+def removeprefix(text: str, prefix: str) -> str:
+    r"""
+    Remove `prefix` from start of `text` if present.
+
+    Backport of `str.removeprefix` for Python versions less than 3.9.
+
+    Args:
+        text: A string to remove the prefix from.
+        prefix: The string to remove from the beginning of `text`.
+    """
+    if text.startswith(prefix):
+        return text[len(prefix):]
+    return text
