@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Copyright (c) 2013, Mahmoud Hashemi
 #
 # Redistribution and use in source and binary forms, with or without
@@ -50,26 +48,11 @@ For more advanced :class:`Table`-style manipulation check out the
 
 """
 
-from __future__ import print_function
 
-try:
-    from html import escape as html_escape
-except ImportError:
-    from cgi import escape as html_escape
+from html import escape as html_escape
 import types
 from itertools import islice
-try:
-    from collections.abc import Sequence, Mapping, MutableSequence
-except ImportError:
-    from collections import Sequence, Mapping, MutableSequence
-try:
-    string_types, integer_types = (str, unicode), (int, long)
-    from cgi import escape as html_escape
-except NameError:
-    # Python 3 compat
-    unicode = str
-    string_types, integer_types = (str, bytes), (int,)
-    from html import escape as html_escape
+from collections.abc import Sequence, Mapping, MutableSequence
 
 try:
     from .typeutils import make_sentinel
@@ -100,12 +83,12 @@ __all__ = ['Table']
 
 def to_text(obj, maxlen=None):
     try:
-        text = unicode(obj)
+        text = str(obj)
     except Exception:
         try:
-            text = unicode(repr(obj))
+            text = str(repr(obj))
         except Exception:
-            text = unicode(object.__repr__(obj))
+            text = str(object.__repr__(obj))
     if maxlen and len(text) > maxlen:
         text = text[:maxlen - 3] + '...'
         # TODO: inverse of ljust/rjust/center
@@ -117,17 +100,17 @@ def escape_html(obj, maxlen=None):
     return html_escape(text, quote=True)
 
 
-_DNR = set((type(None), bool, complex, float,
-            type(NotImplemented), slice,
-            types.FunctionType, types.MethodType, types.BuiltinFunctionType,
-            types.GeneratorType) + string_types + integer_types)
+_DNR = {type(None), bool, complex, float, type(NotImplemented), slice,
+        str, bytes, int,
+        types.FunctionType, types.MethodType,
+        types.BuiltinFunctionType, types.GeneratorType}
 
 
 class UnsupportedData(TypeError):
     pass
 
 
-class InputType(object):
+class InputType:
     def __init__(self, *a, **kw):
         pass
 
@@ -223,7 +206,7 @@ class NamedTupleInputType(InputType):
         return [[getattr(obj, h, None) for h in headers] for obj in obj_seq]
 
 
-class Table(object):
+class Table:
     """
     This Table class is meant to be simple, low-overhead, and extensible. Its
     most common use would be for translation between in-memory data
@@ -437,9 +420,9 @@ class Table(object):
     def __repr__(self):
         cn = self.__class__.__name__
         if self.headers:
-            return '%s(headers=%r, data=%r)' % (cn, self.headers, self._data)
+            return f'{cn}(headers={self.headers!r}, data={self._data!r})'
         else:
-            return '%s(%r)' % (cn, self._data)
+            return f'{cn}({self._data!r})'
 
     def to_html(self, orientation=None, wrapped=True,
                 with_headers=True, with_newlines=True,
