@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import print_function
 import string
 import sys
 sys.path.append('/home/mahmoud/projects/lithoxyl/')
@@ -15,16 +12,9 @@ from collections import OrderedDict as OD
 q_sink = lithoxyl.sinks.QuantileSink()
 log = lithoxyl.logger.BaseLogger('bench_stats', sinks=[q_sink])
 
-try:
-    profile
-except NameError:
-    times = 10
-    size = 10000
-    redun = 2
-else:
-    times = 1
-    size = 10000
-    redun = 10
+times = 10
+size = 10000
+redun = 2
 
 _rng = range(size / redun) * redun
 _unique_keys = set(_rng)
@@ -68,7 +58,7 @@ def bench():
         for action in _all_actions:
             try:
                 best_msecs = q_sink.qas[impl_name][action].min * 1000
-                print('   - %s - %g ms' % (action, best_msecs))
+                print(f'   - {action} - {best_msecs:g} ms')
             except KeyError:
                 pass
         best_msecs = q_sink.qas[impl_name]['total'].min * 1000
