@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 import string
 import sys
 from abc import abstractmethod, ABCMeta
@@ -8,7 +7,7 @@ import pytest
 from boltons.cacheutils import LRU, LRI, cached, cachedmethod, cachedproperty, MinIDMap, ThresholdCounter
 
 
-class CountingCallable(object):
+class CountingCallable:
     def __init__(self):
         self.call_count = 0
 
@@ -167,6 +166,32 @@ def test_lru_basic():
     assert second_lru != lru
 
 
+@pytest.mark.parametrize("lru_class", [LRU, LRI])
+def test_lru_dict_replacement(lru_class):
+    # see issue #348
+    cache = lru_class()
+
+    # Add an entry.
+    cache['a'] = 1
+
+    # Normal __getitem__ access.
+    assert cache['a'] == 1  # passes.
+    # Convert to dict.
+    assert dict(cache) == {'a': 1}  # passes.
+    # Another way to access the only value.
+    assert list(cache.values())[0] == 1  # passes.
+
+    # Replace the existing 'a' entry with a new value.
+    cache['a'] = 200
+
+    # __getitem__ works as expected.
+    assert cache['a'] == 200  # passes.
+
+    # Both dict and accessing via values() return the old entry: 1.
+    assert dict(cache) == {'a': 200}  # fails.
+    assert list(cache.values())[0] == 200
+
+
 def test_lru_with_dupes():
     SIZE = 2
     lru = LRU(max_size=SIZE)
@@ -275,7 +300,7 @@ def test_callable_cached_dec():
 
 
 def test_cachedmethod():
-    class Car(object):
+    class Car:
         def __init__(self, cache=None):
             self.h_cache = LRI() if cache is None else cache
             self.door_count = 0
@@ -352,7 +377,7 @@ def test_cachedmethod_maintains_func_abstraction():
 
 
 def test_cachedproperty():
-    class Proper(object):
+    class Proper:
         def __init__(self):
             self.expensive_func = CountingCallable()
 
@@ -404,7 +429,7 @@ def test_min_id_map():
 
     midm = MinIDMap()
 
-    class Foo(object):
+    class Foo:
         def __init__(self, val):
             self.val = val
 
